/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2012, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//ComputerInfo.java
//Copyright (c) Kawan Softwares S.A.S, 2012
//
//Last Updates: 
// 3 juin 2011 20:09:43 Nicolas de Pomereu
// 08/06/11 10:45 NDP : ComputerInfo: clean trap of exceptions on getMacAddress();
// 18/07/11 19:05 NDP : ComputerInfo: moved into org.awakefw.file.api.util
// 16/02/12 15:15 NDP : ComputerInfo: move to org.awakefw.commons.api.server.util & clean Javadoc. 
// 21/02/12 18:00 NDP : ServerInfo: new name of ComputerInfo

package org.awakefw.commons.api.server.util;

import java.net.InetAddress;
import java.net.NetworkInterface;

import org.awakefw.file.servlet.RequestInfoStore;

/**
 * 
 * Gets some server info, including info extracted from <code>HttpServletRequest</code> at server startup.
 * Includes:
 * <ul>
 * <li>Computer Hostname.</li>
 * <li>Computer MAC Address.</li>
 * <li>Computer IP Address.</li>
 * <li>Server URL (without the port) in <code>http(s)://www.acme.org</code> format.</li>
 * <li>Server scheme.</li>
 * <li>Server port.</li>
 * </ul>
 * 
 * @author Nicolas de Pomereu.
 */
public class ServerInfo {
    /** The hostname */
    private static String hostName = null;

    /** The MAC address */
    private static String macAddress = null;

    /** The IP address */
    private static String ipAddress = null;

    /**
     * Protected
     */
    protected ServerInfo() {
    }

    /**
     * Returns the computer name (Hostname).
     * 
     * @return the name or <b>unknown_hostname</b> if the name cannot be found
     */

    public static String getHostname() {
	try {
	    if (hostName == null) {
		final InetAddress addr = InetAddress.getLocalHost();
		hostName = new String(addr.getHostName());

		if (hostName == null) {
		    hostName = "unknown_hostname";
		}
	    }
	} catch (final Exception e) {
	    hostName = "unknown_hostname";
	    e.printStackTrace(System.out);
	}

	return hostName;
    }

    /**
     * Returns the computer IP address in 192.168.1.146 format.
     * 
     * @return the name or <b>unknown_ip_address</b> if the MAC address cannot
     *         be found
     */

    public static String getIpAddress() {
	try {
	    if (ipAddress == null) {
		InetAddress ip = InetAddress.getLocalHost();

		ipAddress = ip.getHostAddress();
	    }
	} catch (Exception e) {
	    ipAddress = "unknown_ip_address";
	    e.printStackTrace(System.out);
	}

	return ipAddress;
    }

    /**
     * Returns the computer MAC address in 5C-26-0A-88-4E-DA format.
     * 
     * @return the name or <b>unknown_mac_address</b> if the MAC address cannot
     *         be found
     */

    public static String getMacAddress() {
	try {
	    if (macAddress == null) {
		InetAddress ip = InetAddress.getLocalHost();

		NetworkInterface network = NetworkInterface
			.getByInetAddress(ip);

		if (network == null) {
		    macAddress = "unknown_mac_address";
		    return macAddress;
		}

		byte[] mac = network.getHardwareAddress();

		if (mac == null) {
		    macAddress = "unknown_mac_address";
		    return macAddress;
		}

		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < mac.length; i++) {
		    sb.append(String.format("%02X%s", mac[i],
			    (i < mac.length - 1) ? "-" : ""));
		}

		macAddress = sb.toString();
	    }
	} catch (Exception e) {
	    macAddress = "unknown_mac_address";
	    e.printStackTrace(System.out);
	}

	return macAddress;
    }

    /**
     * Server URL (without the port) in <code>http(s)://www.acme.org</code> format.</li>
     * <br><br>
     * Info extracted from <code>HttpServletRequest</code> at server startup.
     * 
     * @return Server URL (without the port) 
     */
    public static String getServerUrl() {
	return new RequestInfoStore().getServerUrl();
    }

    /**
     * Returns the name of the scheme used to make this request, for example,
     * http, https, or ftp. Different schemes have different rules for
     * constructing URLs, as noted in RFC 1738.
     * <br><br>
     * Info extracted from <code>HttpServletRequest</code> at server startup.
     * 
     * @return a String containing the name of the scheme used to make this
     *          request
     */
    public static String getScheme() {
	return new RequestInfoStore().getScheme();
    }

    /**
     * Returns the host name of the server to which the request was sent. It is
     * the value of the part before ":" in the Host header value, if any, or the
     * resolved server name, or the server IP address.
     * <br><br>
     * Info extracted from <code>HttpServletRequest</code> at server startup.
     * 
     * @return a String containing the name of the server
     */
    public static String getServerName() {
	return new RequestInfoStore().getServerName();
    }

    /**
     * Returns the port number to which the request was sent. It is the value of
     * the part after ":" in the Host header value, if any, or the server port
     * where the client connection was accepted on.
     * <br><br>
     * Info extracted from <code>HttpServletRequest</code> at server startup.
     *      
     * @return an integer specifying the port number
     */
    public static int getServerPort() {
	return new RequestInfoStore().getServerPort();
    }

}
